@@ -20,7 +20,7 @@ from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
 from ansible.errors import AnsibleParserError
-from ansible.playbook.attribute import Attribute, FieldAttribute
+from ansible.playbook.attribute import FieldAttribute
 from ansible.playbook.base import Base
 from ansible.playbook.become import Become
 from ansible.playbook.conditional import Conditional
@@ -28,26 +28,33 @@ from ansible.playbook.helpers import load_list_of_tasks
 from ansible.playbook.role import Role
 from ansible.playbook.taggable import Taggable
 
+
 class Block(Base, Become, Conditional, Taggable):
 
-    _block  = FieldAttribute(isa='list', default=[], inherit=False)
+    # main block fields containing the task lists
+    _block = FieldAttribute(isa='list', default=[], inherit=False)
     _rescue = FieldAttribute(isa='list', default=[], inherit=False)
     _always = FieldAttribute(isa='list', default=[], inherit=False)
-    _delegate_to = FieldAttribute(isa='list')
+
+    # other fields
+    _delegate_to = FieldAttribute(isa='string')
     _delegate_facts = FieldAttribute(isa='bool', default=False)
-    _any_errors_fatal = FieldAttribute(isa='bool')
+    _name = FieldAttribute(isa='string', default='')
 
     # for future consideration? this would be functionally
     # similar to the 'else' clause for exceptions
-    #_otherwise = FieldAttribute(isa='list')
+    # _otherwise = FieldAttribute(isa='list')
 
     def __init__(self, play=None, parent_block=None, role=None, task_include=None, use_handlers=False, implicit=False):
-        self._play         = play
-        self._role         = role
-        self._parent       = None
-        self._dep_chain    = None
+        self._play = play
+        self._role = role
+        self._parent = None
+        self._dep_chain = None
         self._use_handlers = use_handlers
-        self._implicit     = implicit
+        self._implicit = implicit
+
+        # end of role flag
+        self._eor = False
 
         if task_include:
             self._parent = task_include
@@ -56,6 +63,9 @@ class Block(Base, Become, Conditional, Taggable):
 
         super(Block, self).__init__()
 
+    def __repr__(self):
+        return "BLOCK(uuid=%s)(id=%s)(parent=%s)" % (self._uuid, id(self), self._parent)
+
     def get_vars(self):
         '''
         Blocks do not store variables directly, however they may be a member
@@ -111,8 +121,8 @@ class Block(Base, Become, Conditional, Taggable):
                 loader=self._loader,
                 use_handlers=self._use_handlers,
             )
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading a block", obj=self._ds, orig_exc=e)
 
     def _load_rescue(self, attr, ds):
         try:
@@ -126,23 +136,23 @@ class Block(Base, Become, Conditional, Taggable):
                 loader=self._loader,
                 use_handlers=self._use_handlers,
             )
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading rescue.", obj=self._ds, orig_exc=e)
 
     def _load_always(self, attr, ds):
         try:
             return load_list_of_tasks(
-                ds, 
+                ds,
                 play=self._play,
-                block=self, 
-                role=self._role, 
+                block=self,
+                role=self._role,
                 task_include=None,
-                variable_manager=self._variable_manager, 
-                loader=self._loader, 
+                variable_manager=self._variable_manager,
+                loader=self._loader,
                 use_handlers=self._use_handlers,
             )
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading always", obj=self._ds, orig_exc=e)
 
     def get_dep_chain(self):
         if self._dep_chain is None:
@@ -173,8 +183,9 @@ class Block(Base, Become, Conditional, Taggable):
             return new_task_list
 
         new_me = super(Block, self).copy()
-        new_me._play         = self._play
+        new_me._play = self._play
         new_me._use_handlers = self._use_handlers
+        new_me._eor = self._eor
 
         if self._dep_chain is not None:
             new_me._dep_chain = self._dep_chain[:]
@@ -184,7 +195,7 @@ class Block(Base, Become, Conditional, Taggable):
             new_me._parent = self._parent.copy(exclude_tasks=exclude_tasks)
 
         if not exclude_tasks:
-            new_me.block  = _dupe_task_list(self.block or [], new_me)
+            new_me.block = _dupe_task_list(self.block or [], new_me)
             new_me.rescue = _dupe_task_list(self.rescue or [], new_me)
             new_me.always = _dupe_task_list(self.always or [], new_me)
 
@@ -207,6 +218,7 @@ class Block(Base, Become, Conditional, Taggable):
                 data[attr] = getattr(self, attr)
 
         data['dep_chain'] = self.get_dep_chain()
+        data['eor'] = self._eor
 
         if self._role is not None:
             data['role'] = self._role.serialize()
@@ -234,6 +246,7 @@ class Block(Base, Become, Conditional, Taggable):
                 setattr(self, attr, data.get(attr))
 
         self._dep_chain = data.get('dep_chain', None)
+        self._eor = data.get('eor', False)
 
         # if there was a serialized role, unpack it too
         role_data = data.get('role')
@@ -251,21 +264,10 @@ class Block(Base, Become, Conditional, Taggable):
                 p = TaskInclude()
             elif parent_type == 'HandlerTaskInclude':
                 p = HandlerTaskInclude()
-            p.deserialize(pb_data)
+            p.deserialize(parent_data)
             self._parent = p
             self._dep_chain = self._parent.get_dep_chain()
 
-    def evaluate_conditional(self, templar, all_vars):
-        dep_chain = self.get_dep_chain()
-        if dep_chain:
-            for dep in dep_chain:
-                if not dep.evaluate_conditional(templar, all_vars):
-                    return False
-        if self._parent is not None:
-            if not self._parent.evaluate_conditional(templar, all_vars):
-                return False
-        return super(Block, self).evaluate_conditional(templar, all_vars)
-
     def set_loader(self, loader):
         self._loader = loader
         if self._parent:
@@ -279,9 +281,9 @@ class Block(Base, Become, Conditional, Taggable):
                 dep.set_loader(loader)
 
     def _get_attr_environment(self):
-        return self._get_parent_attribute('environment', extend=True)
+        return self._get_parent_attribute('environment', extend=True, prepend=True)
 
-    def _get_parent_attribute(self, attr, extend=False):
+    def _get_parent_attribute(self, attr, extend=False, prepend=False):
         '''
         Generic logic to get the attribute or parent attribute for a block value.
         '''
@@ -292,18 +294,19 @@ class Block(Base, Become, Conditional, Taggable):
 
             if self._parent and (value is None or extend):
                 try:
-                    parent_value = getattr(self._parent, attr, None)
-                    if extend:
-                        value = self._extend_value(value, parent_value)
-                    else:
-                        value = parent_value
+                    if attr != 'when' or getattr(self._parent, 'statically_loaded', True):
+                        parent_value = getattr(self._parent, attr, None)
+                        if extend:
+                            value = self._extend_value(value, parent_value, prepend)
+                        else:
+                            value = parent_value
                 except AttributeError:
                     pass
             if self._role and (value is None or extend):
                 try:
                     parent_value = getattr(self._role, attr, None)
                     if extend:
-                        value = self._extend_value(value, parent_value)
+                        value = self._extend_value(value, parent_value, prepend)
                     else:
                         value = parent_value
 
@@ -313,7 +316,7 @@ class Block(Base, Become, Conditional, Taggable):
                         for dep in dep_chain:
                             dep_value = getattr(dep, attr, None)
                             if extend:
-                                value = self._extend_value(value, dep_value)
+                                value = self._extend_value(value, dep_value, prepend)
                             else:
                                 value = dep_value
 
@@ -325,12 +328,12 @@ class Block(Base, Become, Conditional, Taggable):
                 try:
                     parent_value = getattr(self._play, attr, None)
                     if extend:
-                        value = self._extend_value(value, parent_value)
+                        value = self._extend_value(value, parent_value, prepend)
                     else:
                         value = parent_value
                 except AttributeError:
                     pass
-        except KeyError as e:
+        except KeyError:
             pass
 
         return value
@@ -346,15 +349,15 @@ class Block(Base, Become, Conditional, Taggable):
             for task in target:
                 if isinstance(task, Block):
                     tmp_list.append(evaluate_block(task))
-                elif task.action == 'meta' \
-                or (task.action == 'include' and task.evaluate_tags([], play_context.skip_tags, all_vars=all_vars)) \
-                or task.evaluate_tags(play_context.only_tags, play_context.skip_tags, all_vars=all_vars):
+                elif (task.action == 'meta' or
+                        (task.action == 'include' and task.evaluate_tags([], play_context.skip_tags, all_vars=all_vars)) or
+                        task.evaluate_tags(play_context.only_tags, play_context.skip_tags, all_vars=all_vars)):
                     tmp_list.append(task)
             return tmp_list
 
         def evaluate_block(block):
             new_block = self.copy(exclude_tasks=True)
-            new_block.block  = evaluate_and_append_task(block.block)
+            new_block.block = evaluate_and_append_task(block.block)
             new_block.rescue = evaluate_and_append_task(block.rescue)
             new_block.always = evaluate_and_append_task(block.always)
             return new_block
@@ -385,3 +388,10 @@ class Block(Base, Become, Conditional, Taggable):
 
         return True
 
+    def get_first_parent_include(self):
+        from ansible.playbook.task_include import TaskInclude
+        if self._parent:
+            if isinstance(self._parent, TaskInclude):
+                return self._parent
+            return self._parent.get_first_parent_include()
+        return None
